////////////////////////////////////////////////////////////////////////////////
//
// System iNtrusion Analysis and Reporting Environment (SNARE)
// - Audit Daemon Header
//
// Audit Daemon Header file
//
////////////////////////////////////////////////////////////////////////////////
// Copyright 1999-2002 InterSect Alliance Pty Ltd
// - http://www.intersectalliance.com/
////////////////////////////////////////////////////////////////////////////////
#ifndef _AUDITD_H_
#define _AUDITD_H_ 1

#define CONFIG_C2_AUDIT 1
// #include <linux/saudit.h>
#include <regex.h>
#include "saudit.h"

#define MAX_AUDITREC PATH_MAX+4096	// How much buffer to reserve for the textual representation of an audit event.
#define IDELIMITER ","			// Internal Delimiter between fields of a token
#define DELIMITER "\t"			// Will probably let the user change this later.
#define CONFIG_FILENAME "/etc/audit/audit.conf"
#define MAX_AUDIT_CONFIG_LINE		8192
#define MAX_USERNAME			256
#define MAX_OPTIONS			256
#define MAX_EVENTNAME			32
#define MAX_USERREG			PATH_MAX
#define MAX_HOSTID			256	// Host identifier - usually the fully qualified hostname.

#define TIMEBUF	64			// linux man page says 26 chars is sufficient


#define AUDIT_ALL	-999

#define CONFIG_AUDITTYPE	1
#define	CONFIG_OBJECTIVES	2
#define CONFIG_EVENTS		3
#define CONFIG_OUTPUT		4
#define CONFIG_DELIVERY		5
#define CONFIG_HOSTID		6
#define CONFIG_REMOTE		7

#define AUDIT_BY_OBJECTIVE	1
#define AUDIT_BY_EVENT		2

#define AUDIT_TO_STDOUT		1
#define AUDIT_TO_FILE		2
#define AUDIT_TO_NETWORK	4
#define AUDIT_TO_SYSLOG		8

#define CRITICALITY_CLEAR	0
#define CRITICALITY_INFO	1
#define CRITICALITY_WARNING 	2
#define CRITICALITY_PRIORITY 	3
#define CRITICALITY_CRITICAL 	4

#define	RETURNCODE_FAILURE	1
#define RETURNCODE_SUCCESS	2
#define RETURNCODE_ANY		3

#define TARGET_EID		2
#define TARGET_GID		2

// signal masks

#define AUDIT_SIGNAL_STOP       0x01
#define AUDIT_SIGNAL_REFRESH    0x02
#define AUDIT_SIGNAL_SUSPEND    0x04
#define AUDIT_SIGNAL_RESUME     0x08

// Linked List
struct _node {
	int event_number;	// The event number this is supposed to match
	int criticality;	// How critical is this particular node
	int returncode;		// return code required.
	int returnflag;		// return FLAG required (success or failure)
				// one of the above must be set.
	int excludeflag;	// Include or exclude users?
	int excludematchflag;	// Include or exclude the primary match
	int notreturnflag;	// returncode == or != ?
	char username[MAX_USERREG];	// Remember, this will be a regular expression.
	char path[PATH_MAX];	// NOTE: this could match either path or destpath
	char options[MAX_OPTIONS];	// Options associated with a audit type (eg: O_RDONLY|O_CREAT)
	regex_t username_regexp;	// regexp pattern for user name compilation and storage
	regex_t path_regexp;	// regexp pattern for path compilation and storage
	regex_t options_regexp;	// regexp pattern for options compilation and storage
	struct _node *next;
};

typedef struct _node Node;

void sig_handler(int);
void sync_signal_handler(void);
void close_outputs(void);
void abort_audit(char *, int);
int safe_read(int, void *, int);

int process_io_class(header_token *);
void calc_open_options(char *, io_class *);
int process_ex_class(header_token *);
int process_pc_class(header_token *);
int process_ch_class(header_token *);
int process_cp_class(header_token *);
int process_su_class(header_token *);
int process_nt_class(header_token *);
int process_ad_class(header_token *);

char *eventcat(char *, char *);
char *va_eventcat(char *, int, ...);
int sendevent(char *);
char *textmode(int, char *, int);
char *passname(int);
char *groupname(int);

int add_user_token(char *, header_token *);
int add_process_token(char *, header_token *);
int add_return_token(char *, header_token *,int);
int add_sequence_token(char *);

char *eventnumber2name(int, char *, int);
int eventname2number(char *);

char *resolved_path(char *, char *, char *);
int resolved_path2(char *, char *);

int read_config_file(void);

int turn_event_on(int);

char *criticalitystring(int);
void trim(char *);
void trimallwhitespace(char *);
char *strip(char *);
int iscomment(char *);
int isfilename(char *);
int isnetwork(char *);
int open_audit_output(char *);
int open_audit_network(char *);
int splitobjective(char *, char *, char *, char *, int *, int *, int *,
		   int *, int *);
char *   getconfstring(char *string, char *file,int length);
int getport(char *string);

int isheader(char *);
int getheader(char *);
int isaudittype(char *);
int getaudittype(char *);
int regmatch(const char *, const char *);
int regmatchi(const char *, const char *);
int geteventswitch(char *);
char *gethostident(char *, char *, int);
char *getfqdn(char *);
int SendToSocket(char *);
void syslogdate(char *sdate, struct tm *cdate);
int issyslog(char *string);
int get_syslog_dest(char *string);

void ProcessObjectives(Node *, header_token *, const char *, const char *,
		       const char *, const char *, const char *, int);
Node *FindNextObjective(Node *, int, const char *, const char *,
			const char *, int, int);

// Some potential linked list stuff.
void CreateLinkedList(void);
void DestroyList(void);
Node *AddToList(int, char *, char *, int, char *, int, int, int, int, int);
void RemoveFromListHead(void);
void RemoveFromList(Node *);
void DestroyNode(Node *);
void ResetCurrentNode(void);
int IsValidItem(void);
Node *GetCurrentItem(void);
void NextItemInList(void);

#endif // _AUDITD_H_
